/***************************************************************************
*   Copyright (C) 2006 by Michael Kaufmann   *
*   michael@enlighter.de   *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
***************************************************************************/
#include "logwidget.h"
#include "openvpnmanager.h"

#include <iostream.h>
#include <klistview.h>
#include <qwidget.h>
#include <ktabwidget.h>
#include <qlayout.h>
#include <klocale.h>

logWidget::logWidget( QWidget* parent, const char* name, WFlags fl )
      : logWidgetBase( parent, name, fl )
{}

logWidget::~logWidget()
{}

void logWidget::reg( openVPNManager * manager )
{
   debug( "reg" );

   QWidget * TabPage = new QWidget( loggingTabs, manager->id() );
   KListBox * listBox = new KListBox( TabPage, "listBox" );
   QGridLayout * TabPageLayout = new QGridLayout( TabPage, 1, 1, 11, 6, "TabPageLayout" );

   TabPageLayout->addWidget( listBox, 0, 0 );
   loggingTabs->addTab( TabPage, manager->id() );

   listBox->insertItem( i18n( "LOGGING FOR %1 STARTED." ).arg( manager->id() ) );
   listBox->setDragAutoScroll( true );
   listBox->setAcceptDrops( true );

   mTabList[ manager->id() ] = TabPage;
   mListBoxList[ manager->id() ] = listBox;

   QObject::connect( manager, SIGNAL ( message( const QString &, const QString & ) ), this, SLOT ( message( const QString &, const QString & ) ) );
}

void logWidget::dereg( openVPNManager * manager )
{
   QObject::disconnect( manager, 0, 0, 0 );

   loggingTabs->removePage( mTabList[ manager->id() ] );

   /* TODO: Do I need to delete them manually ? */
   //   delete mTabList[ manager->id() ];
   mTabList.remove( manager->id() );

   //   delete mListBoxList[ manager->id() ];
   mListBoxList.remove( manager->id() );

   /* Is the QGridLayout deleted automatically? I think so, because It's a child of TabPage */
}

void logWidget::message( const QString & id, const QString & message )
{
   mListBoxList[ id ] ->insertItem( message );
}

void logWidget::debug( const QString & method, const QString & message )
{
#ifdef DEBUG
   QString myMethod ( method );
   static int maxLen = 0;
   maxLen = QMAX( myMethod.length(), maxLen );

   cout << "logWidget::" << myMethod.leftJustify( maxLen ) << " => " << message << endl;
#endif
}

#include "logwidget.moc"

