/***************************************************************************
*   Copyright (C) 2006 by Michael Kaufmann   *
*   michael@enlighter.de   *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
***************************************************************************/
#include "traywindow.h"
#include "openvpnmanager.h"

#include <kiconloader.h>
#include <iostream.h>
#include <klocale.h>
#include <qtooltip.h>

TrayWindow * TrayWindow::trayWindowInstance = 0L;

TrayWindow * TrayWindow::systemTray( QWidget *parent, const char *name )
{
   if ( ! trayWindowInstance ) {
      trayWindowInstance = new TrayWindow( parent, name );
   }
   trayWindowInstance->setPixmap( trayWindowInstance->loadIcon( "kovpn_disconnected" ) );
   return trayWindowInstance;
}

TrayWindow::TrayWindow( QWidget *parent = 0, const char *name = 0 )
      : KSystemTray( parent, name )
{
   setInactive();
   trayIcons = new KIconLoader( "kovpn" );
   mParent = parent;
   /* Get context menu */
   mContextMenu = contextMenu();
   mDisconnectMenu = new KPopupMenu( mContextMenu );
   mConnectMenu = new KPopupMenu( mContextMenu );

   QObject::connect( this, SIGNAL ( quitSelected() ), parent, SLOT ( slotQuitKovpn() ) );

   /* Fill context menu */
   mPreferencesMenuItem = KStdAction::preferences( parent, SLOT( slotPreferences() ), actionCollection() );
   mPreferencesMenuItem->plug( mContextMenu, 1 );

   mStatusMenuItem = new KAction( i18n( "&State" ), "slotState", 0, parent, SLOT( show() ), actionCollection(), "state" );
   mStatusMenuItem->setIcon( "klinkstatus" );
   mStatusMenuItem->plug( mContextMenu, 1 );

   mLogWidgetMenuItem = new KAction( i18n( "&Messages" ), "slotMessages", 0, parent, SLOT( slotShowLogWidget() ), actionCollection(), "message" );
   mLogWidgetMenuItem->setIcon( "message" );
   mLogWidgetMenuItem->plug( mContextMenu, 1 );

   mContextMenu->insertSeparator( 4 );
}


TrayWindow::~TrayWindow()
{}

void TrayWindow::slotStatusChanged( openVPNManager * manager )
{
   switch ( manager->status() ) {
         case openVPNManager::CONNECTED:
         trayWindowInstance->setPixmap( trayIcons->loadIcon( "kovpn_connected", KIcon::Small ) );
         break;
         case openVPNManager::DISCONNECTED:
         trayWindowInstance->setPixmap( trayIcons->loadIcon( "kovpn_disconnected", KIcon::Small ) );
         break;
         default:
         trayWindowInstance->setPixmap( trayIcons->loadIcon( "kovpn_connecting", KIcon::Small ) );
         break;
   }
}

void TrayWindow::dereg( openVPNManager * manager )
{
   mConnectMenuItems[ manager->id() ] ->unplugAll();
   delete mConnectMenuItems[ manager->id() ];

   mDisconnectMenuItems[ manager->id() ] ->unplugAll();
   delete mDisconnectMenuItems[ manager->id() ];

   /* I think (int) manager is unique and static enough to be used as id for the menu */
   mContextMenu->removeItem( ( int ) manager );
   mManagerMenus.remove( manager->id() );
   mOvpnManagers.remove( manager );
}

void TrayWindow::reg( openVPNManager * manager )
{
   debug( "reg", QString( "Registering %1" ).arg( manager->id() ) );

   QObject::connect( manager, SIGNAL ( signalOvpnStatusChanged( const QString &, openVPNManager::states ) ), this, SLOT ( stateChanged( const QString &, openVPNManager::states ) ) );
   QObject::connect( manager, SIGNAL ( signalOvpnReady( const QString & ) ), this, SLOT ( ovpnReady( const QString & ) ) );
   QObject::connect( manager, SIGNAL ( signalOvpnNotReady( const QString & ) ), this, SLOT ( ovpnNotReady( const QString & ) ) );
   QObject::connect( manager, SIGNAL ( signalOvpnPreReady( const QString & ) ), this, SLOT ( ovpnPreReady( const QString & ) ) );

   mOvpnManagers.append( manager );

   mManagerMenus[ manager->id() ] = new KPopupMenu( mContextMenu );
   /* I think (int) manager is unique and static enough to be used as id for the menu */
   mContextMenu->insertItem( manager->id() , mManagerMenus[ manager->id() ] , ( int ) manager, 5 );

   mManagerMenus[ manager->id() ] ->insertTitle( i18n( "Status: Unavailable" ), 0 );

   mDisconnectMenuItems[ manager->id() ] = new KAction( i18n( "&Disconnect" ), "disconnect", 0, manager, SLOT( disconnect() ), actionCollection(), "disconnect" );
   mDisconnectMenuItems[ manager->id() ] ->plug( mManagerMenus[ manager->id() ], 1 );
   mDisconnectMenuItems[ manager->id() ] ->setIcon( "kovpn_disconnected" );
   mDisconnectMenuItems[ manager->id() ] ->setEnabled( false );

   mConnectMenuItems[ manager->id() ] = new KAction( i18n( "&Connect" ), "connect", 0, manager, SLOT( connect() ), actionCollection(), "connect" );
   mConnectMenuItems[ manager->id() ] ->plug( mManagerMenus[ manager->id() ], 1 );
   mConnectMenuItems[ manager->id() ] ->setIcon( "kovpn_connected" );
   mConnectMenuItems[ manager->id() ] ->setEnabled( false );

   /* Set initial state */
}

void TrayWindow::setState( TrayWindow::states state )
{
   switch ( state ) {
         case DISCONNECTED:
         trayWindowInstance->setPixmap( trayIcons->loadIcon( "kovpn_disconnected", KIcon::Small ) );
         break;
         case HALF_CONNECTED:
         trayWindowInstance->setPixmap( trayIcons->loadIcon( "kovpn_halfconnected", KIcon::Small ) );
         break;
         case CONNECTED:
         trayWindowInstance->setPixmap( trayIcons->loadIcon( "kovpn_connected", KIcon::Small ) );
         break;
         case CONNECTING:
         trayWindowInstance->setPixmap( trayIcons->loadIcon( "kovpn_connecting", KIcon::Small ) );
         break;
         case DISCONNECTED__CONNECTING:
         trayWindowInstance->setPixmap( trayIcons->loadIcon( "kovpn_connecting", KIcon::Small ) );
         break;
         case HALF_CONNECTED__CONNECTING:
         trayWindowInstance->setPixmap( trayIcons->loadIcon( "kovpn_connecting", KIcon::Small ) );
         break;
         case CONNECTED__CONNECTING:
         trayWindowInstance->setPixmap( trayIcons->loadIcon( "kovpn_connecting", KIcon::Small ) );
         break;
   }
}

void TrayWindow::stateChanged( const QString & id, openVPNManager::states status )
{
   /* Only consider managers that are ready (= they have reported a status yet).
   *  If they become "unready" they are deleted from this map in the function "ovpnNotReady"
   */
   mOvpnStates[ id ] = status;

   setToolTip();
   setStateMenu( id, status );
   setEnableMenu( id, status );
   states mStatus = calcState();

   /* update the Icon */
   setState( mState );
}

void TrayWindow::ovpnPreReady( const QString & id )
{
   mManagerMenus[ id ] ->changeTitle( 0, i18n( "No control access" ) );
   mConnectMenuItems[ id ] ->setEnabled( true );
   mDisconnectMenuItems[ id ] ->setEnabled( true );
}

void TrayWindow::ovpnReady( const QString & id )
{
   mConnectMenuItems[ id ] ->setEnabled( true );
   mDisconnectMenuItems[ id ] ->setEnabled( true );
}

void TrayWindow::ovpnNotReady( const QString & id )
{
   mConnectMenuItems[ id ] ->setEnabled( false );
   mDisconnectMenuItems[ id ] ->setEnabled( false );
   /* Remove this id from the list of states, because I only count the openvpn states which are available,
   *  meaning I only count, if openvpn is running for this id. */
   mOvpnStates.remove( id );

   /* Recalc states, because I changed the state map mOvpnStates */
   states mStatus = calcState();
   /* update the Icon */
   setState( mState );
}

void TrayWindow::setToolTip( )
{
   QString mToolTip;

   openVPNManager * manager;
   for ( unsigned int i = 0; i < mOvpnManagers.count(); i++ ) {
      /* The sollution in the QT Docs doesn't work here. Gives me always the same element. */
      manager = mOvpnManagers.at( i );
      switch ( manager->status() ) {
            case openVPNManager::DISCONNECTED:
            mToolTip += i18n( "<NOBR><B>%1</B> is disconnected</NOBR>" ).arg( manager->id() );
            break;
            case openVPNManager::CONNECTING:
            mToolTip += i18n( "<NOBR><B>%1</B> is connecting</NOBR>" ).arg( manager->id() );
            break;
            case openVPNManager::WAIT:
            mToolTip += i18n( "<NOBR><B>%1</B> is waiting</NOBR>" ).arg( manager->id() );
            break;
            case openVPNManager::AUTH:
            mToolTip += i18n( "<NOBR><B>%1</B> is authenticating</NOBR>" ).arg( manager->id() );
            break;
            case openVPNManager::GET_CONFIG:
            mToolTip += i18n( "<NOBR><B>%1</B> is getting remote config</NOBR>" ).arg( manager->id() );
            break;
            case openVPNManager::ASSIGN_IP:
            mToolTip += i18n( "<NOBR><B>%1</B> is assigning IP address</NOBR>" ).arg( manager->id() );
            break;
            case openVPNManager::ADD_ROUTES:
            mToolTip += i18n( "<NOBR><B>%1</B> is adding routes</NOBR>" ).arg( manager->id() );
            break;
            case openVPNManager::CONNECTED:
            mToolTip += i18n( "<NOBR><B >%1</B> is connected</NOBR>" ).arg( manager->id() );
            break;
            case openVPNManager::RECONNECTING:
            mToolTip += i18n( "<NOBR><B>%1</B> is reconnecting</NOBR>" ).arg( manager->id() );
            break;
            case openVPNManager::EXITING:
            mToolTip += i18n( "<NOBR><B>%1</B> is unavailable</NOBR>" ).arg( manager->id() );
            break;
            case openVPNManager::UNKNOWN:
            default:
            mToolTip += i18n( "<NOBR><B>%1</B> is unavailable</NOBR>" ).arg( manager->id() );
            break;
      }
      if ( manager != mOvpnManagers.last() ) {
         mToolTip += "<BR>";
      }
   }
   QToolTip::remove( this );
   QToolTip::add( this, mToolTip );
}

void TrayWindow::setEnableMenu( const QString & id, openVPNManager::states status )
{
   switch ( status ) {
         case openVPNManager::DISCONNECTED:
         case openVPNManager::CONNECTING:
         case openVPNManager::WAIT:
         case openVPNManager::AUTH:
         case openVPNManager::GET_CONFIG:
         case openVPNManager::ASSIGN_IP:
         case openVPNManager::ADD_ROUTES:
         case openVPNManager::RECONNECTING:
         mConnectMenuItems[ id ] ->setEnabled( true );
         mDisconnectMenuItems[ id ] ->setEnabled( false );
         break;
         case openVPNManager::CONNECTED:
         mConnectMenuItems[ id ] ->setEnabled( false );
         mDisconnectMenuItems[ id ] ->setEnabled( true );
         break;
         case openVPNManager::EXITING:
         mConnectMenuItems[ id ] ->setEnabled( false );
         mDisconnectMenuItems[ id ] ->setEnabled( false );
         break;
         case openVPNManager::UNKNOWN:
         default:
         mConnectMenuItems[ id ] ->setEnabled( true );
         mDisconnectMenuItems[ id ] ->setEnabled( true );
         break;
   }
}

void TrayWindow::setStateMenu( const QString & id, openVPNManager::states status )
{
   switch ( status ) {
         case openVPNManager::DISCONNECTED:
         mManagerMenus[ id ] ->changeTitle( 0, i18n( "Status" ) + ": " + i18n( "Disconnected" ) );
         break;
         case openVPNManager::CONNECTING:
         mManagerMenus[ id ] ->changeTitle( 0, i18n( "Status" ) + ": " + i18n( "Connecting" ) );
         break;
         case openVPNManager::WAIT:
         mManagerMenus[ id ] ->changeTitle( 0, i18n( "Status" ) + ": " + i18n( "Waiting" ) );
         break;
         case openVPNManager::AUTH:
         mManagerMenus[ id ] ->changeTitle( 0, i18n( "Status" ) + ": " + i18n( "Authenticating" ) );
         break;
         case openVPNManager::GET_CONFIG:
         mManagerMenus[ id ] ->changeTitle( 0, i18n( "Status" ) + ": " + i18n( "Getting remote config" ) );
         break;
         case openVPNManager::ASSIGN_IP:
         mManagerMenus[ id ] ->changeTitle( 0, i18n( "Status" ) + ": " + i18n( "Assigning IP address" ) );
         break;
         case openVPNManager::ADD_ROUTES:
         mManagerMenus[ id ] ->changeTitle( 0, i18n( "Status" ) + ": " + i18n( "Adding routes" ) );
         break;
         case openVPNManager::CONNECTED:
         mManagerMenus[ id ] ->changeTitle( 0, i18n( "Status" ) + ": " + i18n( "Connected" ) );
         break;
         case openVPNManager::RECONNECTING:
         mManagerMenus[ id ] ->changeTitle( 0, i18n( "Status" ) + ": " + i18n( "Reconnecting" ) );
         break;
         case openVPNManager::EXITING:
         mManagerMenus[ id ] ->changeTitle( 0, i18n( "Status" ) + ": " + i18n( "Unavailable" ) );
         break;
         case openVPNManager::UNKNOWN:
         default:
         mManagerMenus[ id ] ->changeTitle( 0, i18n( "Status" ) + ": " + i18n( "Unknown" ) );
         break;
   }
}

TrayWindow::states TrayWindow::calcState( )
{
   int tConnected = 0;
   int tConnecting = 0;
   int tDisconnected = 0;
   /*
   These are the states:
    1. DISCONNECTED               - all managers are disconnected.
    2. HALF_CONNECTED             - some managers are connected, some are not.
    3. CONNECTED                  - all managers are connected.
    4. DISCONNECTED__CONNECTING   - no managers is connected, some are connecting.
    5. HALF_CONNECTED__CONNECTING - some managers are disconnected, some are connecting.

   These are the OpenVPN states:
    0. EXITING
    1. DISCONNECTED
    2. CONNECTING
    3. WAIT
    4. AUTH
    5. GET_CONFIG
    6. ASSIGN_IP
    7. ADD_ROUTES
    8. RECONNECTING
    9. CONNECTED
   Relevant groups are:
    1. DISCONNECTED = DISCONNECTED, EXITING
    2. CONNECTING   = CONNECTING, WAIT, AUTH, GET_CONFIG, ASSIGN_IP, ADD_ROUTES, RECONNECTING
    3. CONNECTED    = CONNECTED

    ==>
    1. DISCONNECTED               = DISCONNECTED
    2. HALF_CONNECTED             = DISCONNECTED, CONNECTED
    3. CONNECTED                  = CONNECTED
    4. DISCONNECTED__CONNECTING   = DISCONNECTED, CONNECTING
    5. HALF_CONNECTED__CONNECTING = CONNECTED, DISCONNECTED, CONNECTING
   */

   /* First of all, group them in the 3 groups mentioned above */
   for ( QMap<QString, openVPNManager::states>::Iterator it = mOvpnStates.begin(); it != mOvpnStates.end(); it++ ) {
      if ( it.data() == openVPNManager::CONNECTED )
         tConnected++;
      else if ( it.data() == openVPNManager::DISCONNECTED || it.data() == openVPNManager::EXITING )
         tDisconnected++;
      else
         tConnecting++;
   }

   /* Decide what state we've got */
   if ( ! tDisconnected && ! tConnecting && ! tConnected )
      mState = DISCONNECTED;
   else if ( ! tDisconnected && ! tConnecting && tConnected )
      mState = CONNECTED;
   else if ( ! tDisconnected && tConnecting && ! tConnected )
      mState = CONNECTING;
   else if ( ! tDisconnected && tConnecting && tConnected )
      mState = CONNECTED__CONNECTING;
   else if ( tDisconnected && ! tConnecting && ! tConnected )
      mState = DISCONNECTED;
   else if ( tDisconnected && ! tConnecting && tConnected )
      mState = HALF_CONNECTED;
   else if ( tDisconnected && tConnecting && ! tConnected )
      mState = DISCONNECTED__CONNECTING;
   else if ( tDisconnected && tConnecting && tConnected )
      mState = HALF_CONNECTED__CONNECTING;

   debug( "stateChanged", QString( "tDisconnected : %1" ).arg( tDisconnected ) );
   debug( "stateChanged", QString( "tConnecting   : %1" ).arg( tConnecting ) );
   debug( "stateChanged", QString( "tConnected    : %1" ).arg( tConnected ) );
   debug( "stateChanged", QString( "mState        : %1" ).arg( mState ) );
   return mState;
}

void TrayWindow::debug( const QString & method, const QString & message )
{
#ifdef DEBUG
   QString myMethod ( method );
   static int maxLen = 0;
   maxLen = QMAX( myMethod.length(), maxLen );

   cout << "TrayWindow::" << myMethod.leftJustify( maxLen ) << " => " << message << endl;
#endif
}

#include "traywindow.moc"
